package com.brilliance.isc.funds.fxtfop.service;

import com.brilliance.isc.bo.*;
import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.glemod.service.GlemodService;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FxtfopStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtfop.check.FxtfopValidator;
import com.brilliance.isc.funds.fxtlop.service.FxtpService;
import com.brilliance.isc.funds.fxtfop.check.FxtfopValidator;
import com.brilliance.isc.funds.vo.funds.FxtfopStoreVo;

import com.brilliance.isc.funds.vo.funds.FxtssbStoreVo;
import com.brilliance.isc.mda.dao.PtyratMapper;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.Decimals;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.apache.poi.ss.formula.functions.Today;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Map;
import java.util.Objects;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: FxtfopTransactionServiceImpl
 * @description: 即期结汇 Fxtfop 业务处理
 * @author: huangshunlin
 * @create: 2024-10-08 15:35
 */
@Service("fxtfop.transaction")
public class FxtfopTransactionServiceImpl extends AbstractTransactionService<FxtfopStoreBo> {

    @Resource
    private FxtfopValidator fxtfopValidator;

    @Resource
    private XrtmodService xrtmodService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private FxToolComponent fxToolComponent;


    @Resource
    private TransactionHelpService transactionHelpService;


    @Resource
    private SettleContext settleContext;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private TrnmodService trnmodService;

    @Resource
    private GlemodService glemodService;

    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @Resource
    private PtyratMapper ptyratMapper;

    @Resource
    private SetmodService setmodService;

    @Resource
    private FxtpService fxtpService;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        //serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");

    }


    @Override
    protected void txnSave(FxtfopStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    public void ruleMtabutSavN500(FxtfopStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        Fxdgrp fxdgrp = bo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        fxToolComponent.sav(fxdgrp, BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract(trnmodVo);
        cbsSav(bo, trnmodVo);
    }

    @Override
    protected Map<String, String> txnCheck(FxtfopStoreBo transactionVo) {
        return fxtfopValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtfopStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        Fxd rec = transactionVo.getFxdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFxtyp(rec.getFxtyp());
        bizInfoVo.setAcc(rec.getAcc());
        bizInfoVo.setAcc2(rec.getAcc2());
        bizInfoVo.setDsp(rec.getDsp());
        bizInfoVo.setDsp2(rec.getDsp2());
        bizInfoVo.setTrdint(rec.getTrdint());
        bizInfoVo.setTrdout(rec.getTrdout());
        bizInfoVo.setTrnman(rec.getTrnman());
        bizInfoVo.setRat(rec.getRat());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtfopStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtfopStoreBo init(FxtfopStoreBo fxtfopStoreBo) {
        fxtfopStoreBo.reset();
        init950(fxtfopStoreBo);
        init1000(fxtfopStoreBo);
        return fxtfopStoreBo;
    }

    /**
     * source:fxtfop.0001.script
     */
    public void init1000(FxtfopStoreBo bo) {

        bo.getFxtp().setFrgchk("X");
        bo.getFxdgrp().getCbs().getNom1().setCur(xrtmodService.sysiso());
        bo.getFxdgrp().getRec().setFxtyp("FB");
        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        bo.getFxdgrp().getRec().setUsr(extkey);

        if (bo.getFxdgrp().getRec().getOpndat() ==null ){
            bo.getFxdgrp().getRec().setOpndat(Dates.today());
        }

        if (bo.getFxdgrp().getRec().getValdat() ==null ){
            bo.getFxdgrp().getRec().setValdat(Dates.today());
        }

    }

    private void init950(FxtfopStoreBo fxtfopStoreBo) {

    }

    @Override
    public void beforeSave(FxtfopStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();

        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtfopStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFxdgrp().getRec().getInr());
    }


    /**
     * source:fxtfop.0003.script
     */
    public void cbsSav(FxtfopStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN", trnmodVo.getTrn().getInr());
        Fxacbs cbs = transactionVo.getFxdgrp().getCbs();
        String trninr = transactionHelpService.getTrninr();
        trnmodService.cbsTrnStore("MAXAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), "funds_fxtfop_000000", transactionVo.getFxdgrp().getRec(), trninr);
        trnmodService.cbsTrnStore("NOMAMT", "AMT1", Dates.today(), cbs.getNom1().getCur(), cbs.getNom1().getAmt(), I18nUtil.getMessage("funds_fxtfop_000000"), transactionVo.getFxdgrp().getRec(), trninr);

        if ("FS".equals(transactionVo.getFxdgrp().getRec().getFxtyp()) ||"FB".equals(transactionVo.getFxdgrp().getRec().getFxtyp())){
            trnmodService.cbsTrnStore("OPNAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtfop_000000"), transactionVo.getFxdgrp().getRec(), trninr);
        }

    }


    public String getRef(FxtfopStoreVo fxtfopStoreVo) {
        fxtfopStoreVo.reset();
        PtsptaVo ptsptaVo = fxtfopStoreVo.getFxdgrp().getApl();
        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            return NULLSTR;
        }
        Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
        Bch wrkbch = new Bch();
        wrkbch.setInr(pta.getBranchinr());
        wrkbch = bchService.selectByPrimaryKey(wrkbch);
        if (Objects.isNull(wrkbch)) {
            return NULLSTR;
        }
        SettleContext.getUserSession().setWrkbch(wrkbch);
        Fxdgrp fxdgrp = fxtfopStoreVo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        return fxdgrp.getRec().getOwnref();
    }


    /**
     *   source:fxtfop.0008.script
     */

    public void defaultRateN1000(FxtfopStoreVo vo){
        vo.reset();

        String rmbRat = NULLSTR;
        BigDecimal buyRat = null;
        BigDecimal selRat = null;
        BigDecimal midRat = null;
        BigDecimal buyRat1 = null;
        BigDecimal baseunit = null;
        Cbb max = vo.getFxdgrp().getCbs().getMax();
        Fxd rec = vo.getFxdgrp().getRec();
        Pts pts = vo.getFxdgrp().getApl().getPts();
        if( ! MdaUtils.isEmpty(rec.getFxtyp()) && ! MdaUtils.isEmpty(vo.getFxdgrp().getCbs().getMax().getCur()) )
        {
            /**
             CBSMOD\\XRTMOD.getratCebTim ( FXDGRP\\CBS\\MAX\\CUR, SYSISO, Today, $RmbRat, $BuyRat, $SelRat, $MidRat, $BuyRat1 )

             **/
            Argument<String> rmbRatBox = new Argument<String>();
            Argument<BigDecimal> buyRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> selRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> midRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> buyRat1Box = new Argument<BigDecimal>();
            xrtmodService.getratCebTim ( vo.getFxdgrp().getCbs().getMax().getCur(), vo.getFxdgrp().getCbs().getNom1().getCur(), Dates.today(), rmbRatBox, buyRatBox, selRatBox, midRatBox, buyRat1Box );

            rmbRat = rmbRatBox.value;
            buyRat = buyRatBox.value;
            selRat = selRatBox.value;
            midRat = midRatBox.value;
            buyRat1 = buyRat1Box.value;

            if( ! rec.isModified("midrat") )
            {
                baseunit = xrtmodService.crossRatebas( vo.getFxdgrp().getCbs().getMax().getCur(), vo.getFxdgrp().getCbs().getNom1().getCur() );
                if( MdaUtils.compareTo(baseunit,1)<0 )
                {
                    rec.setMidrat( midRat);
                }
                else
                {
                    rec.setMidrat( Decimals.mul(midRat,baseunit));
                }
            }

        }else{
            rec.setRat(BigDecimal.ZERO);
            rec.setMidrat(BigDecimal.ZERO);
        }

    }


    public void defaultCalculateN1100(FxtfopStoreVo fxtfopStoreVo) {
        fxtfopStoreVo.reset();
        fxtpService.defaultCalculateN1100(fxtfopStoreVo.getFxdgrp(), fxtfopStoreVo.getFxtp());
    }

}
