package com.brilliance.isc.funds.transfer.transaction;

import com.brilliance.isc.funds.bo.funds.FxtfopStoreBo;
import com.brilliance.isc.funds.vo.funds.FxtfopStoreVo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;

/**
 * @program: isc-funds
 * @ClassName: FxtssbStructMapper
 * @description: 即期结汇转换Fxtssb Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 * @author: huangshunlin
 * @create: 2024-10-08 15:27
 */
@Mapper(componentModel = "spring")
public interface FxtfopStructMapper extends IBaseStructMapper<FxtfopStoreBo, FxtfopStoreVo> {
    /**
     * 此行语句一定要写，StructMapper需要
     */
    FxtfopStructMapper INSTANCE = Mappers.getMapper(FxtfopStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    FxtfopStoreBo voTransferToBo(FxtfopStoreVo vo);

    /**
     * Vo追加更新至Bo
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({

    })
    void voUpdateToBo(FxtfopStoreVo vo, @MappingTarget FxtfopStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    FxtfopStoreVo boTransferToVo(FxtfopStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(FxtfopStoreBo bo, @MappingTarget FxtfopStoreVo vo);
}

